@@ -22,7 +22,7 @@ test_that("parameters for the linear regression are recovered as with lm", {
   priors <- get_priors(obs(y ~ x, family = "gaussian"),
     data = d, time = "time"
   )
-  priors$prior <- c("normal(0, 5)", "normal(0, 1)", "exponential(1)")
+  priors$prior <- c("normal(0, 5)", "std_normal()", "exponential(1)")
   fit_dynamite <- dynamite(obs(y ~ x, family = "gaussian"),
     data = d, time = "time", priors = priors,
     chains = 1, iter = 2000, refresh = 500
@@ -88,12 +88,63 @@ test_that("parameters for the gamma glm are recovered as with glm", {
   )
 })
 
+test_that("parameters for poisson mixed model are recovered", {
+  skip_if_not(run_extended_tests)
+  set.seed(1)
+  n <- 40
+  k <- 10
+  x <- rnorm(n * k)
+  u1 <- rep(rnorm(k, sd = 0.2), each = n)
+  u2 <- rep(rnorm(k, sd = 0.1), each = n)
+  y <- rpois(n * k, exp(2 - x + u1 + u2 * x))
+  d <- data.frame(year = 1:n, person = rep(1:k, each = n), y = y, x = x)
+
+  # use default priors of brms (except not totally flat for beta)
+  p <- data.frame(
+    parameter = c(
+      "sigma_nu_y_alpha", "sigma_nu_y_x", "alpha_y", "beta_y_x",
+      "L_nu"
+    ),
+    response = "y",
+    prior = c(
+      "student_t(3, 0, 2.5)", "student_t(3, 0, 2.5)",
+      "student_t(3, 1.9, 2.5)", "normal(0, 1000)", "lkj_corr_cholesky(1)"
+    ),
+    type = c("sigma_nu", "sigma_nu", "alpha", "beta", "L"),
+    category = ""
+  )
+  fit_dynamite <- dynamite(
+    obs(y ~ x + random(~ 1 + x), family = "poisson") +
+      random_spec(noncentered = FALSE, correlated = TRUE),
+    data = d, time = "year", group = "person", priors = p,
+    chains = 1, iter = 2000, refresh = 0, seed = 1
+  )
+  # "ground truth" obtained from one long dynamite run and brms,
+  # note that brms can give few divergences as does dynamite if
+  # noncentered = TRUE
+  expect_equal(coef(fit_dynamite)$mean, c(2.014, -0.9932),
+    tolerance = 0.1
+  )
+  expect_equal(coef(fit_dynamite, type = "nu")$mean,
+    c(
+      0.1635, 0.4153, -0.0924, -0.1344, -0.0773, -0.1237, -0.2027,
+      -0.1231, 0.2726, -0.1071, -0.03, 4e-04, 0.0997, -0.1146, -0.0313,
+      0.0146, 0.0407, -0.0169, -0.1407, 0.1635
+    ),
+    tolerance = 0.1
+  )
+})
+
 test_that("parameters for an AR(1) model are recovered as with arima", {
   skip_if_not(run_extended_tests)
   set.seed(1)
   fit <- dynamite(obs(LakeHuron ~ 1, "gaussian") + lags(),
     data = data.frame(LakeHuron, time = seq_len(length(LakeHuron)), id = 1),
-    "id", "time", chains = 1, iter = 2000, refresh = 500
+    time = "time",
+    group = "id",
+    chains = 1,
+    iter = 2000,
+    refresh = 500
   )
   fit_arima <- arima(LakeHuron, c(1, 0, 0))
   expect_equal(coef(fit)$mean[2], coef(fit_arima)[1],
@@ -112,16 +163,21 @@ test_that("LOO works for AR(1) model", {
   set.seed(1)
   fit <- dynamite(obs(LakeHuron ~ 1, "gaussian") + lags(),
     data = data.frame(LakeHuron, time = seq_len(length(LakeHuron)), id = 1),
-    "id", "time", chains = 1, iter = 2000, refresh = 500
+    time = "time",
+    group = "id",
+    chains = 1,
+    iter = 2000,
+    refresh = 500
   )
   l <- loo(fit)
   expect_equal(l$estimates,
-    structure(c(
-      -107.877842970846, 2.86041434691809, 215.755685941693,
-      7.36848739076899, 0.561813071004331, 14.736974781538
-    ),
-    dim = 3:2,
-    dimnames = list(c("elpd_loo", "p_loo", "looic"), c("Estimate", "SE"))
+    structure(
+      c(
+        -107.877842970846, 2.86041434691809, 215.755685941693,
+        7.36848739076899, 0.561813071004331, 14.736974781538
+      ),
+      dim = 3:2,
+      dimnames = list(c("elpd_loo", "p_loo", "looic"), c("Estimate", "SE"))
     ),
     tolerance = 1
   )
@@ -134,7 +190,11 @@ test_that("LFO works for AR(1) model", {
   set.seed(1)
   fit <- dynamite(obs(LakeHuron ~ 1, "gaussian") + lags(),
     data = data.frame(LakeHuron, time = seq_len(length(LakeHuron)), id = 1),
-    "id", "time", chains = 1, iter = 2000, refresh = 500
+    time = "time",
+    group = "id",
+    chains = 1,
+    iter = 2000,
+    refresh = 500
   )
   l <- lfo(fit, L = 20)
   expect_equal(l$ELPD, -90.4188604974201, tolerance = 1)
@@ -186,8 +246,8 @@ test_that("parameters of a time-varying gaussian model are recovered", {
   # compile model only once
   code <- get_code(dformula,
     data = d$data,
-    group = "id",
-    time = "time"
+    time = "time",
+    group = "id"
   )
   model <- rstan::stan_model(model_code = code)
 
@@ -213,7 +273,7 @@ test_that("parameters of a time-varying gaussian model are recovered", {
   # test with a single large dataset
   d <- create_data(T_ = 500, N = 500, D = 100)
   data <- get_data(obs(y ~ -1 + z + varying(~x), family = "gaussian") +
-    splines(df = 100), group = "id", time = "time", data = d$data)
+      splines(df = 100), time = "time", group = "id", data = d$data)
   fit_long <- rstan::sampling(model,
     data = data,
     refresh = 0, chains = 1, iter = 2000,
@@ -229,12 +289,12 @@ test_that("prior parameters are recovered with zero observations", {
   skip_if_not(run_extended_tests)
   set.seed(1)
   d <- data.frame(y = rep(NA, 10), x = rnorm(10), id = 1, time = 1:10)
-  p <- get_priors(obs(y ~ x, "gaussian"), d, "id", "time")
+  p <- get_priors(obs(y ~ x, "gaussian"), d, time = "time", group = "id")
   p$prior[] <- c("normal(2, 0.1)", "normal(5, 0.5)", "exponential(10)")
   fit_prior <- dynamite(obs(y ~ x, "gaussian"),
     data = d,
-    group = "id",
     time = "time",
+    group = "id",
     priors = p,
     iter = 55000,
     warmup = 5000,
@@ -269,3 +329,101 @@ test_that("prior parameters are recovered with zero observations", {
     tolerance = 0.1, ignore_attr = TRUE
   )
 })
+
+test_that("predict recovers correct estimates", {
+  skip_if_not(run_extended_tests)
+  set.seed(1)
+  N <- 20
+  T_ <- 30
+  y <- matrix(0, N, T_)
+  nu <- rnorm(N)
+  y[, 1] <- rbinom(N, size = 1, prob = 0.5)
+  for(t in 2:T_) y[, t] <- rbinom(N, 1, plogis(nu + y[, t-1]))
+
+  ## check these if tests fail ##
+  # model <- rstan::stan_model("testmodel.stan")
+  # fit <- rstan::sampling(model, data = list(N = N, T = T_, y = y), chains = 1,
+  #   iter = 2e4, warmup = 1000)
+  # rstan_obs_results_id1_time4 <- rstan::summary(fit, "y_rep[1, 4]",
+  #   use_cache = FALSE)$summary[, 1:3]
+  # rstan_obs_results_avg4 <- setNames(c(rstan::summary(fit,
+  #   c("mean_y[4]", "sd_y[4]"),
+  #   use_cache = FALSE)$summary[, 1:3]),
+  # c("mean_m", "mean_s", "se_m", "se_s", "sd_m", "sd_s"))
+  #
+  # rstan_prob_results_id1_time4 <- rstan::summary(fit, "y_m[1, 4]",
+  #   use_cache = FALSE)$summary[, 1:3]
+  # rstan_prob_results_avg4 <- setNames(c(rstan::summary(fit,
+  #   c("mean_y_m[4]", "sd_y_m[4]"),
+  #   use_cache = FALSE)$summary[, 1:3]),
+  #   c("mean_m", "mean_s", "se_m", "se_s", "sd_m", "sd_s"))
+
+  rstan_obs_results_id1_time4 <- c(mean = 0.6098, se_mean = 0.0035, sd = 0.4878)
+  rstan_obs_results_avg4 <- c(mean_m = 0.7136, mean_s = 0.4508,
+    se_m = 7e-04, se_s = 4e-04, sd_m = 0.0939, sd_s = 0.0511)
+  rstan_prob_results_id1_time4 <- c(mean = 0.6062, se_mean = 9e-04, sd = 0.1409)
+  rstan_prob_results_avg4 <- c(mean_m = 0.7138, mean_s = 0.213, se_m = 2e-04,
+    se_s = 2e-04, sd_m = 0.0264, sd_s = 0.025)
+
+  d <- data.frame(y = c(y), time = rep(1:T_, each = N), id = 1:N)
+  p <- get_priors(obs(y ~ lag(y) + random(~1), "bernoulli"),
+    data = d, time = "time", group = "id")
+  p$prior[] <- "std_normal()"
+  fitd <- dynamite(obs(y ~ lag(y) + random(~1), "bernoulli"),
+    data = d, time = "time", group = "id", priors = p,
+    chains = 1, iter = 2e4, warmup = 1000, refresh = 0)
+
+  pred <- predict(fitd)
+  y_new <- pred$y_new[pred$time == 4 & pred$id == 1]
+  expect_equal(
+    c(
+      mean = mean(y_new),
+      se_mean = sd(y_new) / sqrt(length(y_new)),
+      sd = sd(y_new)
+    ),
+    rstan_obs_results_id1_time4,
+    tolerance = 0.05
+  )
+
+  res <- pred |>
+    dplyr::filter(time == 4) |>
+    dplyr::group_by(.draw) |>
+    dplyr::summarise(m = mean(y_new), s = sd(y_new)) |>
+    dplyr::summarise(
+      mean_m = mean(m), mean_s = mean(s),
+      se_m = sd(m) / sqrt(dplyr::n()), se_s = sd(s) / sqrt(dplyr::n()),
+      sd_m = sd(m), sd_s = sd(s)
+      ) |> unlist()
+
+  expect_equal(res,
+    rstan_obs_results_avg4,
+    tolerance = 0.01
+  )
+
+  pred_m <- predict(fitd, type = "mean")
+  y_mean <- pred_m$y_mean[pred_m$time == 4 & pred_m$id == 1]
+  expect_equal(
+    c(
+      mean = mean(y_mean),
+      se_mean = sd(y_mean) / sqrt(length(y_mean)),
+      sd = sd(y_mean)
+    ),
+    rstan_prob_results_id1_time4,
+    tolerance = 0.01
+  )
+
+  res <- pred_m |>
+    dplyr::filter(time == 4) |>
+    dplyr::group_by(.draw) |>
+    dplyr::summarise(m = mean(y_mean), s = sd(y_mean)) |>
+    dplyr::summarise(
+      mean_m = mean(m), mean_s = mean(s),
+      se_m = sd(m) / sqrt(dplyr::n()), se_s = sd(s) / sqrt(dplyr::n()),
+      sd_m = sd(m), sd_s = sd(s)
+    ) |> unlist()
+
+  expect_equal(res,
+    rstan_prob_results_avg4,
+    tolerance = 0.01
+  )
+})
